﻿using Bridge;
using Bridge.Html5;
using System;

namespace Cbdx.Tests
{
    public static class App
    {
        public static void Main()
        {
            Document.Body.InnerHTML = "Bind...ExternalDataExchangeDispatcher";
            var externalDataExchangeDispatcher = Script.Get<ExternalDataExchangeDispatcher>(nameof(Cbdx));

            Document.Body.InnerHTML = "Bind...DataExchangeServiceFactory";
            var dataExchangeServiceFactory = new DataExchangeServiceFactory(externalDataExchangeDispatcher);

            Document.Body.InnerHTML = "Bind...IWindowControlService";
            var windowControlService = dataExchangeServiceFactory.Create<IWindowControlService>();

            Document.Body.InnerHTML = "Bind...ICefDevService";
            var cefDevService = dataExchangeServiceFactory.Create<ICefDevService>();

            Document.Body.InnerHTML = "";

            new HTMLDivElement()
                .With(p => p.TextContent = "msg:")
                .With(p => p.OwnerDocument.Body.AppendChild(p));

            var label = new HTMLDivElement()
                .With(p => p.TextContent = "...")
                .With(p => p.OwnerDocument.Body.AppendChild(p));

            Window.OnResize += delegate { label.TextContent = windowControlService.GetWindowState().ToString(); };

            new HTMLInputElement { Type = InputType.Button, Value = "Reload" }
                .With(p => p.OnClick += delegate { windowControlService.Reload(); })
                .With(p => Document.Body.AppendChild(p));

            new HTMLInputElement { Type = InputType.Button, Value = "F12" }
                .With(p => p.OnClick += delegate { cefDevService.ShowF12(); })
                .With(p => Document.Body.AppendChild(p));

            new HTMLInputElement { Type = InputType.Button, Value = "DragMove" }
                .With(p => p.OnMouseDown += delegate { windowControlService.DragMove(); })
                .With(p => Document.Body.AppendChild(p));

            new HTMLInputElement { Type = InputType.Button, Value = "Max" }
                .With(p => p.OnClick += delegate { windowControlService.SetWindowState(WindowState.Maximized); })
                .With(p => Document.Body.AppendChild(p));

            new HTMLInputElement { Type = InputType.Button, Value = "Min" }
                .With(p => p.OnClick += delegate { windowControlService.SetWindowState(WindowState.Minimized); })
                .With(p => Document.Body.AppendChild(p));

            new HTMLInputElement { Type = InputType.Button, Value = "Restore" }
                .With(p => p.OnClick += delegate { windowControlService.SetWindowState(WindowState.Normal); })
                .With(p => Document.Body.AppendChild(p));

            new HTMLInputElement { Type = InputType.Button, Value = "Close" }
                .With(p => p.OnClick += delegate { windowControlService.CloseWindow(); })
                .With(p => Document.Body.AppendChild(p));

            var count = 0;
            new HTMLInputElement { Type = InputType.Button, Value = "Change title" }
                .With(p => p.OnClick += delegate { windowControlService.SetWindowTitle($"Title Changed {count++}"); })
                .With(p => Document.Body.AppendChild(p));
        }

        private static T With<T>(this T me, Action<T> action)
        {
            action(me);
            return me;
        }
    }
}